package com.cmeeting.job;

import cn.chatbot.openai.completion.chat.ChatCompletionRequest;
import cn.chatbot.openai.completion.chat.ChatMessage;
import cn.chatbot.openai.completion.chat.ChatMessageRole;
import cn.chatbot.openai.completion.chat.Message;
import cn.chatbot.openai.service.LLMService;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.cmeeting.dto.UserDTO;
import com.cmeeting.email.EmailSender;
import com.cmeeting.log.service.ProcessLogService;
import com.cmeeting.mapper.primary.MeetingInfoMapper;
import com.cmeeting.mapper.primary.MeetingRecordTemplateMapper;
import com.cmeeting.pojo.MeetingInfo;
import com.cmeeting.pojo.MeetingRecordTemplate;
import com.cmeeting.pojo.UserId;
import com.cmeeting.service.MeetingInfoService;
import com.cmeeting.util.MinioUtils;
import com.deepoove.poi.XWPFTemplate;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.dataformat.xml.XmlMapper;
import com.tencentcloudapi.wemeet.Client;
import com.tencentcloudapi.wemeet.core.authenticator.AuthenticatorBuilder;
import com.tencentcloudapi.wemeet.core.authenticator.JWTAuthenticator;
import com.tencentcloudapi.wemeet.service.meetings.api.MeetingsApi;
import com.tencentcloudapi.wemeet.service.meetings.model.V1MeetingsMeetingIdParticipantsGet200Response;
import com.tencentcloudapi.wemeet.service.meetings.model.V1MeetingsMeetingIdParticipantsGet200ResponseParticipantsInner;
import com.tencentcloudapi.wemeet.service.records.api.RecordsApi;
import com.tencentcloudapi.wemeet.service.records.model.V1AddressesRecordFileIdGet200Response;
import com.tencentcloudapi.wemeet.service.records.model.V1AddressesRecordFileIdGet200ResponseAiMeetingTranscriptsInner;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.xwpf.extractor.XWPFWordExtractor;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Service;

import java.io.*;
import java.math.BigInteger;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.security.SecureRandom;
import java.text.MessageFormat;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

@Data
@NoArgsConstructor
@Slf4j
@Service
public class FileProcessTask {
    private List<String> recordFileIdList;
    private String meetingId;
    private String subMeetingId;
    private String savePath;
    private Map<String, Object> metadata;

    private int retryCount = 0;
    private static final int MAX_RETRY = 3;

    private String tencentAppId;
    private String tencentSdkId;
    private String tencentSecretId;
    private String tencentSecretKey;
    private String tencentAdminUserId;
    private String llmApiAddr;
    private Boolean finalRetry; //表示是兜底重试机制

    private MeetingInfoMapper meetingInfoMapper;
    private MinioUtils minioUtils;
    private EmailSender emailSender;
    private MeetingRecordTemplateMapper meetingRecordTemplateMapper;
    private ProcessLogService processLogService;

    // 实际处理逻辑
    public void process() {
        boolean isSuccess = false;
        while (retryCount <= MAX_RETRY && !isSuccess) {
            Client client = new Client.Builder()
                    .withAppId(tencentAppId).withSdkId(tencentSdkId)
                    .withSecret(tencentSecretId,tencentSecretKey)
                    .build();
            try {
                MeetingInfo meetingInfo = meetingInfoMapper.selectOne(new LambdaQueryWrapper<MeetingInfo>()
                        .eq(MeetingInfo::getMeetingId,meetingId)
                        .eq(subMeetingId != null, MeetingInfo::getSubMeetingId, subMeetingId));

                //每场会议可能会分段录制，查出每个文件的转录记录后拼接
                StringBuffer recordTextBuffer = new StringBuffer();
                for (String recordFileId : recordFileIdList) {
                    //查询录制转写详情
                    RecordsApi.ApiV1AddressesRecordFileIdGetRequest addressRequest =
                            new RecordsApi.ApiV1AddressesRecordFileIdGetRequest.Builder(recordFileId)
                                    .operatorId(tencentAdminUserId)
                                    .operatorIdType("1")
                                    .build();
                    RecordsApi.ApiV1AddressesRecordFileIdGetResponse addressResponse =
                            client.records().v1AddressesRecordFileIdGet(addressRequest,
                                    new JWTAuthenticator.Builder().nonce(BigInteger.valueOf(Math.abs((new SecureRandom()).nextInt())))
                                            .timestamp(String.valueOf(System.currentTimeMillis() / 1000L)));
                    // 处理响应
                    if (addressResponse != null && addressResponse.getData() != null) {
                        log.info("Successfully got address for record file {}", recordFileId);
                        V1AddressesRecordFileIdGet200Response addressData = addressResponse.getData();

                        // 获取AI会议转录文件
                        List<V1AddressesRecordFileIdGet200ResponseAiMeetingTranscriptsInner> transcripts =
                                addressData.getAiMeetingTranscripts();

                        if (transcripts != null && !transcripts.isEmpty()) {
                            log.info("Found {} AI meeting transcripts for record file {}",
                                    transcripts.size(), recordFileId);

                            // 处理每个转录文件
                            for (V1AddressesRecordFileIdGet200ResponseAiMeetingTranscriptsInner transcript : transcripts) {
                                String fileType = transcript.getFileType();
                                String downloadUrl = transcript.getDownloadAddress();

                                if ("txt".equalsIgnoreCase(fileType)) {
                                    log.info("AI Transcript - Type: {}, URL: {}", fileType, downloadUrl);

                                    // 1. 下载文件
                                    byte[] fileData = downloadFile(downloadUrl);

                                    // 2. 将二进制文件转换为文本
                                    String recordTextContent = new String(fileData);
    //                            byte[] fileData = Files.readAllBytes(Paths.get("/20250514132555-中集AIGC项目例会-转写智能优化版.txt"));
    //                            String recordTextContent = new String(fileData);
                                    if(StringUtils.isNotEmpty(recordTextContent.replaceAll("\\n","").trim())){
                                        recordTextBuffer.append("\n\n");
                                        recordTextBuffer.append(recordTextContent);
                                    }
                                }
                            }
                        } else {
                            log.info("No AI meeting transcripts found for record file {}", recordFileId);
                        }
                    } else {
                        log.warn("Empty response for record file: {}", recordFileId);
                    }
                }
                if(StringUtils.isEmpty(recordTextBuffer.toString().replaceAll("\\n","").trim())){
                    log.info("获取的转录文本为空，跳过纪要生成，meetingId:{}，fileRecordId:{}",meetingId,recordFileIdList.toString());
                    processLogService.log(meetingId,subMeetingId,"获取的转录文本为空，跳过纪要生成");
                    throw new RuntimeException("获取的转录文本为空，跳过纪要生成");
                }
                // 3. 处理文件 (调用Claude API等)
                String choiceTemplateType = choiceTemplateType(meetingInfo.getSubject(),recordTextBuffer.toString());
                log.info("choiceTemplateType->{}",choiceTemplateType);
                String processedResult = processWithClaude(recordTextBuffer.toString());

                // 4. 保存结果
                saveResult(savePath, processedResult, recordTextBuffer.toString().getBytes(StandardCharsets.UTF_8),meetingInfo);

                isSuccess = true;
            } catch (Exception e) {
                StringWriter sw = new StringWriter();
                PrintWriter pw = new PrintWriter(sw);
                e.printStackTrace(pw);
                processLogService.log(meetingId,subMeetingId,sw.toString());
                // 异常处理
                retryCount++;
                if (retryCount > MAX_RETRY) {
                    log.error("达到最大重试次数:meetingId {}", meetingId);
                    //如果是兜底重试，最终还是失败了，设置会议的重试状态为已重试
                    if(finalRetry){
                        meetingInfoMapper.update(null,
                                new LambdaUpdateWrapper<MeetingInfo>()
                                        .eq(MeetingInfo::getMeetingId,meetingId)
                                        .eq(subMeetingId != null,MeetingInfo::getSubMeetingId,subMeetingId)
                                        .set(MeetingInfo::getGenerateRetry,Boolean.TRUE));
                    }
                }else{
                    // 指数退避
                    try {
                        Thread.sleep((long) Math.pow(2, retryCount) * 1000);
                    } catch (InterruptedException ie) {
                        Thread.currentThread().interrupt();
                        throw new RuntimeException("重试失败", ie);
                    }
                }
            }
        }

    }
    
    private byte[] downloadFile(String url) {
        // 实现文件下载逻辑
        OkHttpClient client = new OkHttpClient();
        Request request = new Request.Builder().url(url).build();
        try {
            Response response = client.newCall(request).execute();
            if (!response.isSuccessful()) throw new IOException("Unexpected code " + response);
            return response.body().bytes();
        }catch (Exception e){
            throw new RuntimeException("下载文件失败", e);
        }
    }

    private String getRecordTextContent(byte[] fileData) {
        // 定义文件路径和文件名
        String fullPath = savePath + (System.currentTimeMillis() / 1000L) +  ".docx";
        // 将字节内容写入本地文件
        try (FileOutputStream fos = new FileOutputStream(fullPath)) {
            fos.write(fileData);
            log.info("DOCX transcript saved to: {}", fullPath);
        } catch (IOException e) {
            log.error("Error saving DOCX transcript to file: {}", e.getMessage(), e);
        }
        XWPFDocument document;
        try {
            FileInputStream fis = new FileInputStream(fullPath);
            document = new XWPFDocument(fis);
        } catch (IOException e) {
            throw new RuntimeException(e.getMessage());
        }
        XWPFWordExtractor extractor = new XWPFWordExtractor(document);
        String textContent = extractor.getText();
        log.info("DOCX content as string:\n{}", textContent);
        return textContent;
    }

    /**
     * 提供会议转录文件和会议主题，判断会议类型
     * @param subject
     * @param textContent
     * @return
     */
    private String choiceTemplateType(String subject,String textContent) {
        String token = "AKIAXFAXF62IWJXGLVEE.LnKInaahcMZG9zLsGMH3nTLOw3S3lK5Vcu0+ifnO";
        String apiAddr = llmApiAddr + "/llm/sse-invoke";
        String model = "anthropic.claude-3-5-sonnet-20240620-v1:0";
        int maxTokens = 5000;
        String prompt = "请先对以下会议转写记录进行简要总结（不超过200字），然后根据会议主题以及总结内容判断该会议最可能属于哪种类型：\\n\\n会议主题： {0}\\n会议转写记录： {1}\\n\\n第一步：请简要总结会议的主要内容和目的（不超过200字）。\\n\\n第二步：根据上述总结和会议主题，将会议分类为以下类型之一：\\n1. 项目沟通会 - 与具体项目进展、问题讨论相关的会议\\n2. 重要会议 - 高层决策、战略规划等重要会议\\n3. 启动会 - 项目启动、活动筹备等初始会议\\n4. 其他 - 不符合以上任何分类\\n\\n分类权重判断规则：\\n- 基础权重分配：会议内容总结(70%)、会议主题(30%)\\n- 会议主题权重动态调整：\\n  * 如果会议主题包含明确分类关键词（如\\\"启动会\\\"、\\\"项目沟通\\\"、\\\"战略决策\\\"等），则会议主题权重提升至60%，总结内容权重调整为40%\\n  * 如果会议主题过于简单或模糊（如仅包含\\\"沟通\\\"、\\\"讨论\\\"、\\\"会议\\\"等通用词），则会议主题权重降低至15%，总结内容权重提升至85%\\n- 一致性判断：如果会议主题和总结内容指向不同分类，优先采用内容总结的分类，除非会议主题非常明确且规范\\n\\n输出格式：\\n请只返回分类名称(如\\\"项目沟通会\\\")，不要包含其他内容（如生成的会议总结等）。";
        //占位符信息替换
        String formatPrompt = formatMessage(prompt, subject, textContent);
        List<Message> messages = new ArrayList<>();
        ChatMessage chatMessage = new ChatMessage(ChatMessageRole.USER.value(), formatPrompt);
        messages.add(chatMessage);

        // 调用Claude API处理文件
        String ret = call_llm(apiAddr, model, token, messages, maxTokens);
        return ret;
    }

    private String formatMessage(String pattern, Object... args) {
        return MessageFormat.format(pattern, args);
    }
    
    private String processWithClaude(String textContent) {
        //将文件传送给大模型处理
        String token = "AKIAXFAXF62IWJXGLVEE.LnKInaahcMZG9zLsGMH3nTLOw3S3lK5Vcu0+ifnO";
        String apiAddr = llmApiAddr + "/llm/sse-invoke";
        String model = "anthropic.claude-3-5-sonnet-20240620-v1:0";
        int maxTokens = 5000;
        //获取系统模板
        MeetingRecordTemplate meetingRecordTemplate = meetingRecordTemplateMapper.selectById(1);
        List<Message> messages = new ArrayList<>();
        ChatMessage chatMessage = new ChatMessage(ChatMessageRole.USER.value(), meetingRecordTemplate.getPrompt());
        messages.add(chatMessage);
        chatMessage = new ChatMessage(ChatMessageRole.ASSISTANT.value(), "好的请提供会议记录");
        messages.add(chatMessage);
        chatMessage = new ChatMessage(ChatMessageRole.USER.value(), textContent);

        messages.add(chatMessage);

        // 调用Claude API处理文件
        String ret = call_llm(apiAddr, model, token, messages, maxTokens);
        return ret;
    }

    private void saveResult(String path, String content, byte[] recordData, MeetingInfo meetingInfo) {
        String nowTime = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd-HHmmss"));
        String targetPath = path + nowTime + ".json";  // 改为.json扩展名;
        String targetFileName;
        String meetingName;
        String xml = extractXmlFromMarkdown(content);

        // 将xml格式的内容转换为特殊json
        String json = convertXmlToJSON(xml);
        String recordContentPath = meetingId + "-recordContent-" + IdUtil.fastSimpleUUID() + "txt";
        String recordXmlPath = meetingId + "-recordXmlPath-" + IdUtil.fastSimpleUUID() + "xml";
        try {
            minioUtils.upload(recordContentPath,recordData);
            //写入json文件，这里的json文件用于用户可编辑
            Files.write(Paths.get(targetPath), json.getBytes());
            log.info("json文件已保存到: {}", targetPath);
            //将xml格式的内容转换为map,用于填充模板
            Map<String, Object> dataModel = convertXmlToMap(xml);
            //追加参会人员信息
            Map<String,Object> participantsMap = new ConcurrentHashMap<>();
            DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            String meetingDate = meetingInfo.getStartTime().toLocalDate().format(df);
            participantsMap.put("meeting_date",meetingDate);
            participantsMap.put("meeting_location","线上腾讯会议");
            participantsMap.put("meeting_participants", meetingInfo.getParticipantUsers());
            participantsMap.put("meeting_host",meetingInfo.getHost());

            dataModel.putAll(participantsMap);
            ClassPathResource resource = new ClassPathResource("template/data_net_template.docx");
            meetingName = dataModel.get("meeting_name") != null ? String.valueOf(dataModel.get("meeting_name")) : "腾讯会议纪要";
            targetFileName = meetingName + "_" + nowTime;
            XWPFTemplate template;
            try (InputStream inputStream = resource.getInputStream()) {
                template = XWPFTemplate.compile(inputStream).render(dataModel);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
            template.writeAndClose(new FileOutputStream(path + targetFileName + ".docx"));
            byte[] recordXmlData = Files.readAllBytes(Paths.get(path + targetFileName + ".docx"));
            minioUtils.upload(recordXmlPath,recordXmlData);
            processLogService.log(meetingId,subMeetingId,"填充会议纪要成功");
        } catch (Exception e) {
            log.error("填充会议纪要失败: {}", e.getMessage(), e);
            StringWriter sw = new StringWriter();
            PrintWriter pw = new PrintWriter(sw);
            e.printStackTrace(pw);
            processLogService.log(meetingId,subMeetingId,"填充会议纪要失败"+sw.toString());
            throw new RuntimeException("填充会议纪要失败");
        }

        Boolean isPushed;
        log.info("开始邮件推送------");
        if(meetingInfo.getEmailPushAccess()){
            log.info("用户允许邮件推送，准备推送邮件至{}------",meetingInfo.getEmail());
            //邮件推送
            isPushed = emailSender.sendEmailWithAttachment("duanxincheng@chatbot.cn",meetingName,path + targetFileName + ".docx",meetingId);
            try {
//                isPushed = emailSender.sendEmailWithAttachment(meetingInfo.getEmail(),meetingName,path + targetFileName + ".docx",meetingId);
            } catch (Exception e) {
                log.error("邮件推送失败: {}", e.getMessage(), e);
                StringWriter sw = new StringWriter();
                PrintWriter pw = new PrintWriter(sw);
                e.printStackTrace(pw);
                processLogService.log(meetingId,subMeetingId,"【邮件推送】："+sw.toString());
                throw new RuntimeException(e);
            }
            if(isPushed)
                processLogService.log(meetingId,subMeetingId,"用户允许邮件推送，推送邮件至"+meetingInfo.getEmail());
//        emailSender.sendEmailWithAttachment("xuwentao@chatbot.cn",meetingName,path + targetFileName + ".docx",meetingId);
//        emailSender.sendEmailWithAttachment("jiaqi.cai@cimc.com",meetingName,path + targetFileName + ".docx",meetingId);
        }else{
            log.info("用户关闭了邮件推送，推送终止------");
            processLogService.log(meetingId,subMeetingId,"用户关闭了邮件推送，推送终止");
            isPushed = Boolean.FALSE;
        }

        meetingInfoMapper.update(meetingInfo,
                new LambdaUpdateWrapper<MeetingInfo>()
                        .eq(MeetingInfo::getMeetingId,meetingId)
                        .eq(subMeetingId != null,MeetingInfo::getSubMeetingId,subMeetingId)
                        .set(MeetingInfo::getRecordContent,recordContentPath)
                        .set(MeetingInfo::getRecordXml,recordXmlPath)
                        .set(MeetingInfo::getIsGenerated,Boolean.TRUE)
                        .set(MeetingInfo::getIsPushed,isPushed)
        );
    }

    private String convertXmlToJSON(String xml) {
        String json;
        try {
            XmlMapper xmlMapper = new XmlMapper();
            JsonNode rootNode = xmlMapper.readTree(xml.getBytes());

            // 正确获取节点和属性的方式
            List<Map> list = new ArrayList<>();
            Iterator<String> iterator = rootNode.fieldNames();
            while (iterator.hasNext()){
                String tagName = iterator.next();
                JsonNode subNode = rootNode.path(tagName);
                String displayName = subNode.path("label").asText();
                String content = subNode.path("").asText();
                list.add(new HashMap(){{
                    put("key",tagName + "/" + displayName);
                    put("value",content);
                }});
            }
            // 构建JSON对象
            ObjectMapper jsonMapper = new ObjectMapper();
            json = jsonMapper.writeValueAsString(list);
        } catch (IOException e) {
            log.error(e.getMessage());
            throw new RuntimeException(e.getMessage());
        }
        return json;
    }

    private String call_llm(String apiAddr, String model, String token, List<Message> messages, int maxTokens) {
        LLMService service = new LLMService(token, apiAddr);
        StringBuilder stringBuilder = new StringBuilder();
        try {
            ChatCompletionRequest chatCompletionRequest = ChatCompletionRequest.builder()
                    .model(model)
                    .messages(messages)
                    .stream(true)
                    .n(1)
                    .maxTokens(maxTokens)
                    .logitBias(new HashMap<>())
                    .build();

            service.streamChatCompletion(chatCompletionRequest).doOnError(Throwable::printStackTrace).blockingForEach(chunk -> {
                chunk.getChoices().stream().map(choice -> choice.getMessage().getContent())
                        .filter(Objects::nonNull).findFirst().ifPresent(o -> {
                    try {
                        stringBuilder.append(new String(o.getBytes(Charset.defaultCharset())));
                    } catch (Exception e) {
                        throw new RuntimeException(e);
                    }
                });
            });
        } catch (Exception e) {
            StringWriter sw = new StringWriter();
            PrintWriter pw = new PrintWriter(sw);
            e.printStackTrace(pw);
            processLogService.log(meetingId,subMeetingId,"【大模型处理异常】:"+sw.toString());
            throw new RuntimeException(e);
        }
        service.shutdownExecutor();
        return stringBuilder.toString();
    }
    public static Map<String, Object> convertXmlToMap(String xml) throws Exception {

        XmlMapper xmlMapper = new XmlMapper();
        ObjectMapper objectMapper = new ObjectMapper();

        // 先将 XML 读取为 Map
        Map<?, ?> xmlMap = xmlMapper.readValue(xml, Map.class);

        // 转换为更标准的 Map<String, Object>
        Map<String,Object> map = objectMapper.convertValue(xmlMap, Map.class);
        //特殊处理map格式
        for (Map.Entry<String, Object> entry : map.entrySet()) {
            Map<String,Object> value = (Map<String, Object>) entry.getValue();
            Object realValue = value.get("");
            entry.setValue(realValue);
        }
        return map;
    }

    /**
     * markdown转xml
     * @param markdown
     * @return
     */
    private static String extractXmlFromMarkdown(String markdown) {
        StringBuffer sb = null;
        try {
            int start = markdown.indexOf("<");
            int end = markdown.lastIndexOf(">") + 1;
            sb = new StringBuffer();
            sb.append("<root>");
            sb.append(markdown.substring(start, end).trim());
            sb.append("</root>");
        } catch (Exception e) {
            log.info("markdown转xml，markdown->{}",markdown);
            throw new RuntimeException(e.getMessage());
        }
        return sb.toString();
    }


    public FileProcessTask(List<String> recordFileIdList, String meetingId, String subMeetingId, String savePath, Map<String, Object> metadata, String tencentAppId,
                           String tencentSdkId, String tencentSecretId, String tencentSecretKey, String tencentAdminUserId,
                           MeetingInfoMapper meetingInfoMapper, MinioUtils minioUtils, EmailSender emailSender, MeetingRecordTemplateMapper meetingRecordTemplateMapper,
                           String llmApiAddr, Boolean finalRetry, ProcessLogService processLogService) {
        this.recordFileIdList = recordFileIdList;
        this.savePath = savePath;
        this.metadata = metadata;
        this.tencentAppId = tencentAppId;
        this.tencentSdkId = tencentSdkId;
        this.tencentSecretId = tencentSecretId;
        this.tencentSecretKey = tencentSecretKey;
        this.tencentAdminUserId = tencentAdminUserId;
        this.meetingId = meetingId;
        this.subMeetingId = subMeetingId;
        this.meetingInfoMapper = meetingInfoMapper;
        this.minioUtils = minioUtils;
        this.emailSender = emailSender;
        this.meetingRecordTemplateMapper = meetingRecordTemplateMapper;
        this.llmApiAddr = llmApiAddr;
        this.finalRetry = finalRetry;
        this.processLogService = processLogService;
    }
}